"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthenticationServiceComponent = void 0;
const tslib_1 = require("tslib");
const body_parser_1 = tslib_1.__importDefault(require("body-parser"));
const cookie_parser_1 = tslib_1.__importDefault(require("cookie-parser"));
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const authorization_1 = require("@bleco/authorization");
const core_2 = require("@loopx/core");
const multi_tenancy_1 = require("@loopx/multi-tenancy");
const aliaser_1 = require("./aliaser");
const controllers_1 = require("./controllers");
const enums_1 = require("./enums");
const keys_1 = require("./keys");
const models_1 = require("./models");
const auth_1 = require("./modules/auth");
const keys_2 = require("./modules/auth/keys");
const keycloak_logout_provider_1 = require("./modules/auth/providers/keycloak-logout.provider");
const keycloak_verify_provider_1 = require("./modules/auth/providers/keycloak-verify.provider");
const providers_1 = require("./providers");
const auth_code_generator_provider_1 = require("./providers/auth-code-generator.provider");
const default_role_provider_1 = require("./providers/default-role.provider");
const default_tenant_provider_1 = require("./providers/default-tenant.provider");
const keys_3 = require("./providers/keys");
const mfa_provider_1 = require("./providers/mfa.provider");
const otp_email_sender_provider_1 = require("./providers/otp-email-sender.provider");
const otp_sms_sender_provider_1 = require("./providers/otp-sms-sender.provider");
const repositories_1 = require("./repositories");
const sequence_1 = require("./sequence");
const services_1 = require("./services");
let AuthenticationServiceComponent = exports.AuthenticationServiceComponent = class AuthenticationServiceComponent {
    constructor(application, mfaConfig, otpConfig, authConfig, config) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.application = application;
        this.mfaConfig = mfaConfig;
        this.otpConfig = otpConfig;
        this.authConfig = authConfig;
        this.config = config;
        aliaser_1.ConfigAliaser.apply(application);
        this.bindings = [];
        this.providers = {};
        if (!this.application.isBound(`${core_1.CoreBindings.COMPONENTS}.${core_2.CoreComponent.name}`)) {
            // Mount core component
            this.application.component(core_2.CoreComponent);
        }
        if (+((_a = process.env.AZURE_AUTH_ENABLED) !== null && _a !== void 0 ? _a : 0)) {
            const expressMiddlewares = (_b = this.application.getSync(core_2.LxCoreBindings.EXPRESS_MIDDLEWARES)) !== null && _b !== void 0 ? _b : [];
            expressMiddlewares.push((0, cookie_parser_1.default)());
            expressMiddlewares.push(body_parser_1.default.urlencoded({ extended: true }));
            this.application.bind(core_2.LxCoreBindings.EXPRESS_MIDDLEWARES).to(expressMiddlewares);
        }
        // Mount authentication component
        this.setupAuthenticationComponent((_c = this.config) === null || _c === void 0 ? void 0 : _c.secureClient);
        this.setupMultiFactorAuthentication();
        // Mount authorization component
        this.setupAuthorizationComponent();
        // Mount logout providers
        this.setupLogoutProviders();
        // Mount MultiTenancy component
        this.setupMultiTenancyComponent();
        this.application.api({
            openapi: '3.0.0',
            info: {
                title: 'Authentication Service',
                version: '1.0.0',
            },
            paths: {},
            components: {
                securitySchemes: core_2.SECURITY_SCHEME_SPEC,
            },
            servers: [{ url: '/' }],
        });
        // Mount default sequence if needed
        if (!(authConfig === null || authConfig === void 0 ? void 0 : authConfig.useCustomSequence)) {
            // Mount default sequence if needed
            this.setupSequence();
        }
        this.application
            .bind(providers_1.AuthEntityBindings.DefaultTenantKey)
            .to((_e = (_d = authConfig === null || authConfig === void 0 ? void 0 : authConfig.defaultTenantKey) !== null && _d !== void 0 ? _d : process.env.DEFAULT_TENANT_KEY) !== null && _e !== void 0 ? _e : 'default');
        this.application
            .bind(providers_1.AuthEntityBindings.DefaultRoleName)
            .to((_g = (_f = authConfig === null || authConfig === void 0 ? void 0 : authConfig.defaultRoleKey) !== null && _f !== void 0 ? _f : process.env.DEFAULT_ROLE_NAME) !== null && _g !== void 0 ? _g : 'default');
        if (!this.application.isBound(rest_1.RestBindings.ERROR_WRITER_OPTIONS)) {
            this.application.bind(rest_1.RestBindings.ERROR_WRITER_OPTIONS).to({
                safeFields: ['code', 'errorCode', 'data'],
            });
        }
        this.controllers = (0, core_2.matchResources)(controllers_1.controllers, authConfig === null || authConfig === void 0 ? void 0 : authConfig.controllers);
        this.application.bind(keys_1.AuthServiceBindings.ActorIdKey).to('userId');
        this.application.bind(keys_1.AuthServiceBindings.MarkUserActivity).to({ markUserActivity: false });
        this.repositories = repositories_1.repositories;
        this.services = services_1.services;
        this.models = models_1.models;
        // TODO: session middleware
    }
    /**
     * Setup ServiceSequence by default if no other sequence provided
     */
    setupSequence() {
        this.application.sequence(sequence_1.MySequence);
    }
    setupAuthenticationComponent(secureClient = false) {
        // Add authentication component
        this.application.component(authentication_1.AuthenticationComponent);
        // Customize authentication verify handlers
        if (!secureClient) {
            this.providers[authentication_1.Strategies.Passport.OAUTH2_CLIENT_PASSWORD_VERIFIER.key] = auth_1.ClientPasswordVerifyProvider;
            this.providers[authentication_1.Strategies.Passport.RESOURCE_OWNER_PASSWORD_VERIFIER.key] = auth_1.ResourceOwnerVerifyProvider;
        }
        else {
            this.providers[authentication_1.Strategies.Passport.OAUTH2_CLIENT_PASSWORD_VERIFIER.key] = auth_1.SecureClientPasswordVerifyProvider;
            this.providers[authentication_1.Strategies.Passport.RESOURCE_OWNER_PASSWORD_VERIFIER.key] = auth_1.SecureResourceOwnerVerifyProvider;
        }
        this.providers[authentication_1.Strategies.Passport.BEARER_TOKEN_VERIFIER.key] = auth_1.BearerTokenVerifyProvider;
        this.providers[authentication_1.Strategies.Passport.LOCAL_PASSWORD_VERIFIER.key] = auth_1.LocalPasswordVerifyProvider;
        this.providers[authentication_1.Strategies.Passport.GOOGLE_OAUTH2_VERIFIER.key] = auth_1.GoogleOauth2VerifyProvider;
        this.providers[authentication_1.Strategies.Passport.APPLE_OAUTH2_VERIFIER.key] = auth_1.AppleOauth2VerifyProvider;
        this.providers[authentication_1.Strategies.Passport.FACEBOOK_OAUTH2_VERIFIER.key] = auth_1.FacebookOauth2VerifyProvider;
        this.providers[authentication_1.Strategies.Passport.COGNITO_OAUTH2_VERIFIER.key] = auth_1.CognitoOauth2VerifyProvider;
        this.providers[authentication_1.Strategies.Passport.KEYCLOAK_VERIFIER.key] = keycloak_verify_provider_1.KeycloakVerifyProvider;
        this.providers[authentication_1.Strategies.Passport.AUTHA_VERIFIER.key] = auth_1.AuthaVerifyProvider;
        this.providers[keys_3.SignUpBindings.KEYCLOAK_SIGN_UP_PROVIDER.key] = providers_1.KeyCloakSignupProvider;
        this.providers[keys_3.SignUpBindings.GOOGLE_SIGN_UP_PROVIDER.key] = providers_1.GoogleOauth2SignupProvider;
        this.providers[keys_3.SignUpBindings.INSTAGRAM_SIGN_UP_PROVIDER.key] = providers_1.InstagramOauth2SignupProvider;
        this.providers[keys_3.SignUpBindings.APPLE_SIGN_UP_PROVIDER.key] = providers_1.AppleOauth2SignupProvider;
        this.providers[keys_3.SignUpBindings.FACEBOOK_SIGN_UP_PROVIDER.key] = providers_1.FacebookOauth2SignupProvider;
        this.providers[keys_3.SignUpBindings.COGNITO_SIGN_UP_PROVIDER.key] = providers_1.CognitoOauth2SignupProvider;
        this.providers[keys_3.SignUpBindings.LOCAL_SIGNUP_PROVIDER.key] = providers_1.LocalSignupProvider;
        this.providers[keys_3.SignUpBindings.PRE_LOCAL_SIGNUP_PROVIDER.key] = providers_1.LocalPreSignupProvider;
        this.providers[keys_3.SignUpBindings.SIGNUP_HANDLER_PROVIDER.key] = providers_1.SignupTokenHandlerProvider;
        this.providers[keys_3.SignUpBindings.AUTHA_SIGNUP_PROVIDER.key] = providers_1.AuthaSignupProvider;
        this.providers[keys_3.VerifyBindings.KEYCLOAK_PRE_VERIFY_PROVIDER.key] = providers_1.KeyCloakPreVerifyProvider;
        this.providers[keys_3.VerifyBindings.KEYCLOAK_POST_VERIFY_PROVIDER.key] = providers_1.KeyCloakPostVerifyProvider;
        this.providers[keys_3.VerifyBindings.GOOGLE_PRE_VERIFY_PROVIDER.key] = providers_1.GooglePreVerifyProvider;
        this.providers[keys_3.VerifyBindings.GOOGLE_POST_VERIFY_PROVIDER.key] = providers_1.GooglePostVerifyProvider;
        this.providers[keys_3.VerifyBindings.INSTAGRAM_PRE_VERIFY_PROVIDER.key] = providers_1.InstagramPreVerifyProvider;
        this.providers[keys_3.VerifyBindings.INSTAGRAM_POST_VERIFY_PROVIDER.key] = providers_1.InstagramPostVerifyProvider;
        this.providers[keys_3.VerifyBindings.APPLE_PRE_VERIFY_PROVIDER.key] = providers_1.ApplePreVerifyProvider;
        this.providers[keys_3.VerifyBindings.APPLE_POST_VERIFY_PROVIDER.key] = providers_1.ApplePostVerifyProvider;
        this.providers[keys_3.VerifyBindings.FACEBOOK_PRE_VERIFY_PROVIDER.key] = providers_1.FacebookPreVerifyProvider;
        this.providers[keys_3.VerifyBindings.FACEBOOK_POST_VERIFY_PROVIDER.key] = providers_1.FacebookPostVerifyProvider;
        this.providers[keys_3.VerifyBindings.COGNITO_PRE_VERIFY_PROVIDER.key] = providers_1.CognitoPreVerifyProvider;
        this.providers[keys_3.VerifyBindings.COGNITO_POST_VERIFY_PROVIDER.key] = providers_1.CognitoPostVerifyProvider;
        this.providers[keys_3.VerifyBindings.BEARER_SIGNUP_VERIFY_PROVIDER.key] = providers_1.SignupBearerVerifyProvider;
        this.providers[keys_3.VerifyBindings.AUTHA_PRE_VERIFY_PROVIDER.key] = providers_1.AuthaPreVerifyProvider;
        this.providers[keys_3.VerifyBindings.AUTHA_POST_VERIFY_PROVIDER.key] = providers_1.AuthaPostVerifyProvider;
        this.providers[keys_3.AuthCodeBindings.CODEREADER_PROVIDER.key] = providers_1.OauthCodeReaderProvider;
        this.providers[keys_3.AuthCodeBindings.CODEWRITER_PROVIDER.key] = providers_1.CodeWriterProvider;
        this.providers[keys_3.AuthCodeBindings.AUTH_CODE_GENERATOR_PROVIDER.key] = auth_code_generator_provider_1.AuthCodeGeneratorProvider;
        this.application.bind(authentication_1.AuthenticationBindings.USER_MODEL.key).to(auth_1.AuthUser);
        if (process.env.JWT_PRIVATE_KEY && process.env.JWT_PRIVATE_KEY !== '') {
            this.providers[keys_3.AuthCodeBindings.JWT_SIGNER.key] = providers_1.JWTAsymmetricSignerProvider;
        }
        else {
            this.providers[keys_3.AuthCodeBindings.JWT_SIGNER.key] = providers_1.JWTSymmetricSignerProvider;
        }
        if (process.env.JWT_PRIVATE_KEY && process.env.JWT_PRIVATE_KEY !== '') {
            this.providers[keys_3.AuthCodeBindings.JWT_VERIFIER.key] = providers_1.JWTAsymmetricVerifierProvider;
        }
        else {
            this.providers[keys_3.AuthCodeBindings.JWT_VERIFIER.key] = providers_1.JWTSymmetricVerifierProvider;
        }
        this.providers[keys_1.AuthServiceBindings.JWTPayloadProvider.key] = providers_1.JwtPayloadProvider;
        this.providers[keys_1.AuthServiceBindings.ForgotPasswordHandler.key] = providers_1.ForgotPasswordProvider;
        this.providers[providers_1.AuthEntityBindings.DefaultTenantProvider.key] = default_tenant_provider_1.DefaultTenantProvider;
        this.providers[providers_1.AuthEntityBindings.DefaultRoleProvider.key] = default_role_provider_1.DefaultRoleProvider;
        this.providers[authentication_1.Strategies.Passport.AZURE_AD_VERIFIER.key] = auth_1.AzureAdVerifyProvider;
        this.providers[keys_3.SignUpBindings.AZURE_AD_SIGN_UP_PROVIDER.key] = providers_1.AzureAdSignupProvider;
        this.providers[keys_3.VerifyBindings.AZURE_AD_PRE_VERIFY_PROVIDER.key] = providers_1.AzurePreVerifyProvider;
        this.providers[keys_3.VerifyBindings.AZURE_AD_POST_VERIFY_PROVIDER.key] = providers_1.AzurePostVerifyProvider;
    }
    setupAuthorizationComponent() {
        // Add authorization component
        if (!this.application.isBound(authorization_1.AuthorizationBindings.CONFIG)) {
            this.application.bind(authorization_1.AuthorizationBindings.CONFIG).to({
                allowAlwaysPaths: ['/explorer'],
            });
        }
        this.application.component(authorization_1.AuthorizationComponent);
    }
    setupMultiFactorAuthentication() {
        var _a, _b, _c;
        this.providers[keys_3.VerifyBindings.MFA_PROVIDER.key] = mfa_provider_1.MfaProvider;
        if (((_a = this.mfaConfig) === null || _a === void 0 ? void 0 : _a.secondFactor) === authentication_1.STRATEGY.OTP) {
            if (((_b = this.otpConfig) === null || _b === void 0 ? void 0 : _b.method) === enums_1.OtpMethodType.OTP) {
                this.providers[keys_3.VerifyBindings.OTP_GENERATE_PROVIDER.key] = providers_1.OtpGenerateProvider;
                this.providers[keys_3.VerifyBindings.OTP_SENDER_PROVIDER.key] = providers_1.OtpSenderProvider;
                this.providers[keys_3.VerifyBindings.OTP_EMAIL_SENDER_PROVIDER.key] = otp_email_sender_provider_1.OtpEmailSenderProvider;
                this.providers[keys_3.VerifyBindings.OTP_SMS_SENDER_PROVIDER.key] = otp_sms_sender_provider_1.OtpSmsSenderProvider;
                this.providers[keys_3.VerifyBindings.OTP_PROVIDER.key] = providers_1.OtpProvider;
                this.providers[authentication_1.Strategies.Passport.OTP_VERIFIER.key] = auth_1.OtpVerifyProvider;
                this.providers[keys_3.VerifyBindings.PASSWORDLESS_VERIFIER.key] = auth_1.PasswordlessVerifyProvider;
            }
            else if (((_c = this.otpConfig) === null || _c === void 0 ? void 0 : _c.method) === enums_1.OtpMethodType.GOOGLE_AUTHENTICATOR) {
                this.providers[authentication_1.Strategies.Passport.OTP_VERIFIER.key] = auth_1.GoogleAuthenticatorVerifyProvider;
                this.providers[keys_3.VerifyBindings.PASSWORDLESS_VERIFIER.key] = auth_1.GoogleAuthenticatorVerifyProvider;
            }
            else {
                // do nothing
            }
        }
    }
    setupLogoutProviders() {
        this.providers[keys_2.LogoutBindings.AUTHA_LOGOUT_PROVIDER.key] = auth_1.AuthaLogoutProvider;
        this.providers[keys_2.LogoutBindings.KEYCLOAK_LOGOUT_PROVIDER.key] = keycloak_logout_provider_1.KeycloakLogoutProvider;
    }
    setupMultiTenancyComponent() {
        this.application.configure(multi_tenancy_1.MultiTenancyBindings.ACTION).to({
            strategyNames: ['header', 'jwt', 'query'],
        });
        this.application.component(multi_tenancy_1.MultiTenancyComponent);
        this.providers[multi_tenancy_1.MultiTenancyBindings.TENANT_RESOLVER.key] = providers_1.TenantResolverProvider;
    }
};
exports.AuthenticationServiceComponent = AuthenticationServiceComponent = tslib_1.__decorate([
    (0, core_1.injectable)({
        tags: { [core_1.ContextTags.KEY]: keys_1.AuthServiceBindings.COMPONENT },
    }),
    tslib_1.__param(0, (0, core_1.inject)(core_1.CoreBindings.APPLICATION_INSTANCE)),
    tslib_1.__param(1, (0, core_1.inject)(keys_1.AuthServiceBindings.MfaConfig, { optional: true })),
    tslib_1.__param(2, (0, core_1.inject)(keys_1.AuthServiceBindings.OtpConfig, { optional: true })),
    tslib_1.__param(3, (0, core_1.inject)(keys_1.AuthServiceBindings.Config, { optional: true })),
    tslib_1.__param(4, (0, core_1.inject)(authentication_1.AuthenticationBindings.CONFIG, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [rest_1.RestApplication, Object, Object, Object, Object])
], AuthenticationServiceComponent);
//# sourceMappingURL=component.js.map