"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeycloakLogoutProvider = void 0;
const tslib_1 = require("tslib");
const base_64_1 = require("base-64");
const https_proxy_agent_1 = require("https-proxy-agent");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
const url_1 = require("url");
const context_1 = require("@loopback/context");
const authentication_1 = require("@bleco/authentication");
const proxyUrl = (_a = process.env.HTTPS_PROXY) !== null && _a !== void 0 ? _a : process.env.HTTP_PROXY;
const getProxyAgent = () => {
    if (proxyUrl) {
        return new https_proxy_agent_1.HttpsProxyAgent(proxyUrl);
    }
    return undefined;
};
let KeycloakLogoutProvider = exports.KeycloakLogoutProvider = class KeycloakLogoutProvider {
    constructor(options) {
        this.options = options;
    }
    value() {
        return async (refreshToken) => {
            if (!this.options) {
                throw new Error('Keycloak logout provider is not configured');
            }
            const { host, realm, clientID, clientSecret } = this.options;
            const params = new url_1.URLSearchParams();
            const logoutUrl = `${host}/auth/realms/${realm}/protocol/openid-connect/logout`;
            params.append('refresh_token', refreshToken);
            const strToEncode = `${clientID}:${clientSecret}`;
            await (0, node_fetch_1.default)(logoutUrl, {
                agent: getProxyAgent(),
                method: 'post',
                body: params,
                headers: {
                    Authorization: `Basic ${(0, base_64_1.encode)(strToEncode)}`,
                },
            });
            return undefined;
        };
    }
};
exports.KeycloakLogoutProvider = KeycloakLogoutProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, context_1.inject)(authentication_1.Strategies.Passport.KEYCLOAK_STRATEGY_OPTIONS, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [Object])
], KeycloakLogoutProvider);
//# sourceMappingURL=keycloak-logout.provider.js.map