"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OtpRequest = exports.OtpConnectionUserKeyMap = exports.OtpSupportedConnections = void 0;
exports.OtpSupportedConnections = ['sms', 'email'];
exports.OtpConnectionUserKeyMap = {
    sms: 'phone',
    email: 'email',
};
class OtpRequest {
    constructor(uci, connection, upn, user) {
        this.uci = uci;
        this.conn = connection;
        this.upn = upn;
        this.key = exports.OtpConnectionUserKeyMap[this.conn];
        this.user = user || null;
    }
    static from(uci) {
        const parts = uci.split(':', 2).map(s => s.trim());
        if (parts.length === 1) {
            return;
        }
        const connection = parts[0];
        if (!exports.OtpSupportedConnections.includes(connection)) {
            throw new Error(`Invalid otp request, unsupported connection type: ${connection}`);
        }
        const target = parts[1];
        return new OtpRequest(uci, connection, target);
    }
    static fromUser(user, uci) {
        for (const conn of exports.OtpSupportedConnections) {
            const key = exports.OtpConnectionUserKeyMap[conn];
            if (user[key]) {
                return new OtpRequest(uci, conn, user[key], user);
            }
        }
        throw new Error(`Cannot find any contact (both email and phone number) from user ${user.id}`);
    }
}
exports.OtpRequest = OtpRequest;
//# sourceMappingURL=otp.js.map